import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.TimeZone;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.xml.datatype.DatatypeConfigurationException;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpStatus;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.TrustSelfSignedStrategy;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.util.EntityUtils;
import org.json.JSONObject;


public class TesteRest {

	private static final String URL_BASE = "https://homolog.pen.api.trafficmanager.net/interoperabilidade/rest/v2";
//	private static final String URL_BASE = "https://pen-api.trafficmanager.net/interoperabilidade/rest/v2";
	
	private static int CINQUENTA_MEGAS_EM_BYTES = 52428800;
	
	/**
	 * Método auxiliar para gerar o hash do arquivo em SHA256
	 * 
	 */
	public static String gerarHashSha256(File arquivo) throws FileNotFoundException, IOException {
		byte[] hash = DigestUtils.sha256(new FileInputStream(arquivo));
		return Base64.encodeBase64String(hash);
	}
	
	public static void main(String[] args) throws ClientProtocolException, IOException, KeyManagementException, KeyStoreException, NoSuchAlgorithmException, CertificateException, UnrecoverableKeyException, InvalidKeyException, SignatureException, DatatypeConfigurationException {
		// Trust own CA and all self-signed certs
		KeyStore keyStore = KeyStore.getInstance("pkcs12");
		KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
		keyStore.load(new FileInputStream(".\\arquivos\\ALLYSSON-HOMOLOG.p12"), "1234".toCharArray());
		trustStore.load(new FileInputStream(".\\arquivos\\cacerts"), "changeit".toCharArray());
        SSLContext sslcontext = SSLContexts.custom()
            .loadTrustMaterial(trustStore, new TrustSelfSignedStrategy())
            .loadKeyMaterial(keyStore, "1234".toCharArray())
            .build();
        // Allow TLSv1 protocol only
        SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(
            sslcontext,
            new String[] { "TLSv1" },
            null,
            new HostnameVerifier() {
				public boolean verify(String hostname, SSLSession session) {
					// APENAS PARA TESTES LOCAIS,
					// NÃO USE EM CÓDIGO PRODUTIVO
					return true;
				}
			});
        CloseableHttpClient httpclient = HttpClients.custom()
            .setSSLSocketFactory(sslsf)
            .build();
        
		JSONObject respostaEnvio = enviaProcessoAdministrativo(httpclient);
		long ticket = respostaEnvio.getLong("ticketParaEnvioDeComponentesDigitais");
		long idt = respostaEnvio.getLong("IDT");
				

		uploadArquivoBinario(httpclient, ticket);
		downloadReciboEnvio(httpclient, idt);
//		
//		use para testar um ou mais passos intermediários
//		long ticket = 4;
//		long idt = 4;

		String nre = solicitaMetadados(httpclient, idt);
		downloadBinario(httpclient, idt);
		enviaReciboTramite(httpclient, idt, nre);
		downloadReciboTramite(httpclient, idt);
		
//		outros serviços

//		uploadArquivoBinarioEmPartes(httpclient, ticket, 257049551);
//		terminarEnvio(httpclient, ticket);
//		downloadBinarioEmPartes(httpclient, idt, 257049551);		
//		cancelarTramite(httpclient, 9383);
//		recusaTramite(httpclient, idt);
//      cienciaRecusaTramite(httpclient, idt);
	}

	private static void downloadReciboTramite(CloseableHttpClient httpclient, long idt) throws ClientProtocolException, IOException {
		
		HttpGet httpGet = new HttpGet(URL_BASE + "/tramites/" + idt + "/recibo");
		
		CloseableHttpResponse response = httpclient.execute(httpGet);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Download do recibo: " + strResponse);
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
		
	}

	private static void enviaReciboTramite(CloseableHttpClient httpclient, long idt, String nre) throws SignatureException, DatatypeConfigurationException, NoSuchAlgorithmException, KeyStoreException, CertificateException, FileNotFoundException, IOException, UnrecoverableKeyException, InvalidKeyException {
		
		Calendar agora = Calendar.getInstance();
		
//		DadosDoReciboDeTramite dadosEnvio = new DadosDoReciboDeTramite();
//		
//		dadosEnvio.setIdt(idt);
//		dadosEnvio.setDataDeRecebimento(new Date());
		
		Signature signature = Signature.getInstance("SHA256withRSA");
		
		KeyStore keystore = KeyStore.getInstance("PKCS12");
		keystore.load(new FileInputStream(".\\arquivos\\ALLYSSON-HOMOLOG.p12"), "1234".toCharArray());
		PrivateKey key = (PrivateKey) keystore.getKey("ALLYSSON-HOMOLOG", "1234".toCharArray());
		signature.initSign(key);
		
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");

		// este passo é de EXTREMA importância
		// para que a validação da assinatura
		// do lado do servidor ocorra corretamente
		format.setTimeZone(TimeZone.getTimeZone("UTC"));
		
		StringBuilder conteudoAsString = new StringBuilder();
		conteudoAsString.append("<recibo><IDT>");
		conteudoAsString.append(idt);
		conteudoAsString.append("</IDT><NRE>");
		conteudoAsString.append(nre);
		conteudoAsString.append("</NRE><dataDeRecebimento>");
		conteudoAsString.append(format.format(agora.getTime()));
		conteudoAsString.append("</dataDeRecebimento>");
		
		conteudoAsString.append("<hashDoComponenteDigital>8/jWfz773OQKSMf6JW2yNwgHB7N4hNUskfEi4rjnXK8=</hashDoComponenteDigital>");
		
		conteudoAsString.append("</recibo>");
		byte[] conteudo = conteudoAsString.toString().getBytes();
		
		signature.update(conteudo);
		String assinatura = new String(Base64.encodeBase64(signature.sign()));
		
//		dadosEnvio.setHashDaAssinatura(assinatura);
		
		HttpPost httpPost = new HttpPost(URL_BASE + "/tramites/" + idt + "/recibo");
		httpPost.setHeader("Content-Type", "application/json");
		
		StringBuilder strRequest = new StringBuilder();
		strRequest.append("{");
		strRequest.append("    \"dataDeRecebimento\": \"" + format.format(agora.getTime()) + "\",");
		strRequest.append("    \"hashDaAssinatura\": \"" + assinatura + "\"");
		strRequest.append("}");
		httpPost.setEntity(new StringEntity(strRequest.toString()));
		
		CloseableHttpResponse response = httpclient.execute(httpPost);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Envio do recibo de trâmite.");
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
	}
	
	private static void terminarEnvio(CloseableHttpClient httpclient, long ticket) throws ClientProtocolException, IOException {
		HttpPost httpPost = new HttpPost(URL_BASE + "/tickets-de-envio-de-componente/" + ticket + "/protocolos/componentes-a-enviar/partes/sinalizacao-termino-envio");
		httpPost.setHeader("Content-Type", "application/json");
		
		StringBuilder strRequest = new StringBuilder();
		strRequest.append("{");
		strRequest.append("    \"hashDoComponenteDigital\": \"" + "8/jWfz773OQKSMf6JW2yNwgHB7N4hNUskfEi4rjnXK8=" + "\",");
		strRequest.append("    \"protocolo\": \"" + "123456" + "\"");
		strRequest.append("}");
		httpPost.setEntity(new StringEntity(strRequest.toString()));
		
		
		CloseableHttpResponse response = httpclient.execute(httpPost);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Componente digital: " + strResponse);
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
	}
	
	private static void downloadBinario(CloseableHttpClient httpclient, long idt) throws ClientProtocolException, IOException {
		HttpPost httpPost = new HttpPost(URL_BASE + "/tramites/" + idt + "/protocolos/componentes-digitais");
		httpPost.setHeader("Content-Type", "application/json");
		
		StringBuilder strRequest = new StringBuilder();
		strRequest.append("{");
		strRequest.append("    \"hashDoComponenteDigital\": \"" + "8/jWfz773OQKSMf6JW2yNwgHB7N4hNUskfEi4rjnXK8=" + "\",");
		strRequest.append("    \"protocolo\": \"" + "123456" + "\"");
		strRequest.append("}");
		httpPost.setEntity(new StringEntity(strRequest.toString()));
		
		
		CloseableHttpResponse response = httpclient.execute(httpPost);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Componente digital: " + strResponse);
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
	}

	private static void downloadBinarioEmPartes(CloseableHttpClient httpclient, long idt, int tamanhoBytes) throws ClientProtocolException, IOException {
		double numeroPartesD = (double) tamanhoBytes / CINQUENTA_MEGAS_EM_BYTES;
		
		int numeroPartes = (int)Math.ceil(numeroPartesD);
		int parteInicio = 0;
		int parteFim = CINQUENTA_MEGAS_EM_BYTES;
		
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream( );
		for (int i = 0; i < numeroPartes; i++) {
			HttpPost httpPost = new HttpPost(URL_BASE + "/tramites/" + idt + "/protocolos/componentes-digitais/partes/" + parteInicio + "-" + parteFim);
			httpPost.setHeader("Content-Type", "application/json");
			
			StringBuilder strRequest = new StringBuilder();
			strRequest.append("{");
			strRequest.append("    \"hashDoComponenteDigital\": \"" + "8/jWfz773OQKSMf6JW2yNwgHB7N4hNUskfEi4rjnXK8=" + "\",");
			strRequest.append("    \"protocolo\": \"" + "123456" + "\"");
			strRequest.append("}");
			httpPost.setEntity(new StringEntity(strRequest.toString()));
			
			
			CloseableHttpResponse response = httpclient.execute(httpPost);

			try {
			    System.out.println(response.getStatusLine());
			    
			    String strResponse = EntityUtils.toString(response.getEntity());
			    outputStream.write(Base64.decodeBase64(strResponse));
			    
			    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
			    	System.out.println("Componente digital: parte " + (i+1));
			    } else {
			    	throw new RuntimeException(strResponse);
			    }
			    
			} finally {
			    response.close();
			}
			
			parteInicio = parteFim;
			if (numeroPartes - 2 == i) {
				parteFim = tamanhoBytes;
			}else {
				parteFim += CINQUENTA_MEGAS_EM_BYTES;
			}
		}
		
		File f = new File ("E:\\testes-barramento\\arquivodownload.pdf");
		FileOutputStream fos = null;
		try {
		    fos = new FileOutputStream (f);
		    fos.write (outputStream.toByteArray());
		} finally {
		    if (fos != null) try { fos.close(); } catch (IOException ex) {}
		}
	}
	
	private static void recusaTramite(CloseableHttpClient httpclient, long idt) throws ClientProtocolException, IOException {
		HttpPost httpPost = new HttpPost(URL_BASE + "/tramites/" + idt + "/recusa");
		httpPost.setHeader("Content-Type", "application/json");
		
		StringBuilder strRequest = new StringBuilder();
		strRequest.append("{");
		strRequest.append("    \"justificativa\": \"" + "justifica de teste" + "\",");
		strRequest.append("    \"motivo\": \"" + "99" + "\"");
		strRequest.append("}");
		httpPost.setEntity(new StringEntity(strRequest.toString()));
		
		
		CloseableHttpResponse response = httpclient.execute(httpPost);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Recusa: " + strResponse);
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
	}
	
	private static void cienciaRecusaTramite(CloseableHttpClient httpclient, long idt) throws ClientProtocolException, IOException {
		HttpGet httpGet = new HttpGet(URL_BASE + "/tramites/" + idt + "/ciencia");
		httpGet.setHeader("Content-Type", "application/json");
	
		CloseableHttpResponse response = httpclient.execute(httpGet);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Ciencia: " + strResponse);
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
	}

	private static String solicitaMetadados(CloseableHttpClient httpclient, long idt) throws ClientProtocolException, IOException {
		
		HttpGet httpGet = new HttpGet(URL_BASE + "/tramites/" + idt);
		
		CloseableHttpResponse response = httpclient.execute(httpGet);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Metadados: " + strResponse);
		    	JSONObject jsonObject = new JSONObject(strResponse);
		    	return jsonObject.getString("nre");
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
	}
	
	private static void cancelarTramite(CloseableHttpClient httpclient, long idt) throws ClientProtocolException, IOException {
		
		HttpDelete httpDelete = new HttpDelete(URL_BASE + "/tramites/" + idt);
		
		CloseableHttpResponse response = httpclient.execute(httpDelete);

		try {
			System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("CancelarTramite: " + strResponse);
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		} finally {
		    response.close();
		}
	}

	private static void downloadReciboEnvio(CloseableHttpClient httpclient, long idt) throws ClientProtocolException, IOException {
		
		HttpGet httpGet = new HttpGet(URL_BASE + "/tramites/" + idt + "/recibo-de-envio");
		
		CloseableHttpResponse response = httpclient.execute(httpGet);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Download do recibo de envio: " + strResponse);
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
		
	}
	
	private static void uploadArquivoBinario(CloseableHttpClient httpclient, long ticket) throws ClientProtocolException, IOException {
		String recurso = URL_BASE + "/tickets-de-envio-de-componente/" + ticket + "/protocolos/componentes-a-enviar";
		HttpPut httpPut = new HttpPut(recurso);
		System.out.println(recurso);
		
		MultipartEntityBuilder builder = MultipartEntityBuilder.create();
		builder.addBinaryBody("conteudo", new File("E:\\testes-barramento\\binario1.txt"));
		builder.addTextBody("hashDoComponenteDigital", "8/jWfz773OQKSMf6JW2yNwgHB7N4hNUskfEi4rjnXK8=");
		builder.addTextBody("protocolo", "123456");
		httpPut.setEntity(builder.build());
		
		CloseableHttpResponse response = httpclient.execute(httpPut);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Upload do arquivo binário.");
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
		
	}


	private static void uploadArquivoBinarioEmPartes(CloseableHttpClient httpclient, long ticket, int tamanhoBytes) throws ClientProtocolException, IOException {
		File file = new File("E:\\testes-barramento\\FILE_245MB.pdf");
		FileInputStream fileInput = new FileInputStream(file);
		
		double numeroPartesD = (double) tamanhoBytes / CINQUENTA_MEGAS_EM_BYTES;
		
		int numeroPartes = (int)Math.ceil(numeroPartesD);
		int parteInicio = 0;
		int parteFim = CINQUENTA_MEGAS_EM_BYTES;
		
		byte[][] arquivoBytes = lerParteArquivo(fileInput, numeroPartes, tamanhoBytes);
		for (int i = 0; i < numeroPartes; i++) {
			String recurso = URL_BASE + "/tickets-de-envio-de-componente/" + ticket + "/protocolos/componentes-a-enviar/partes/" + parteInicio + "-" + parteFim;
			HttpPut httpPut = new HttpPut(recurso);
			System.out.println(recurso);
			MultipartEntityBuilder builder = MultipartEntityBuilder.create();
			builder.addBinaryBody("conteudo", arquivoBytes[i]);
			builder.addTextBody("hashDoComponenteDigital", "8/jWfz773OQKSMf6JW2yNwgHB7N4hNUskfEi4rjnXK8=");
			builder.addTextBody("protocolo", "123456");
			httpPut.setEntity(builder.build());
			
			CloseableHttpResponse response = httpclient.execute(httpPut);

			try {
			    System.out.println(response.getStatusLine());
			    
			    String strResponse = EntityUtils.toString(response.getEntity());
			    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
			    	System.out.println("Upload do arquivo binário.");
			    } else {
			    	throw new RuntimeException(strResponse);
			    }
			    
			} finally {
			    response.close();
			}
			
			parteInicio = parteFim;
			if (numeroPartes - 2 == i) {
				parteFim = tamanhoBytes;
			}else {
				parteFim += CINQUENTA_MEGAS_EM_BYTES;
			}
			
		}
	}
	
	private static byte[][] lerParteArquivo(FileInputStream inFile, int partes, int tamanhoBytes) throws IOException {
//		File file = new File("caminho");
		FileOutputStream fileOut = new FileOutputStream("arquivo");
		int inicio = 0;
		int fim = CINQUENTA_MEGAS_EM_BYTES;
		
		byte[][] resultado = new byte[partes][1];
		for (int i = 0; i < partes; i++) {
			byte[] sendBuf = new byte[fim - inicio];
	        inFile.read(sendBuf, 0, sendBuf.length);
	        resultado[i] = sendBuf;
	        
	        inicio = fim;
			if (partes - 2 == i) {
				fim = tamanhoBytes;
			}else {
				fim += CINQUENTA_MEGAS_EM_BYTES;
			}
		}
		
		
//        fileOut.write(sendBuf);
//		for(int x = inicio ; x < fim ; x++ ) {						
//			 int result = fileInput.read();//Le o arquivo		 			  	
//			 escrevre.println(result); //Grava no destino						
//		}
        
        return resultado;
	}


	private static JSONObject enviaProcessoAdministrativo(CloseableHttpClient httpclient) throws ClientProtocolException, IOException {
		
		HttpPost httpPost = new HttpPost(URL_BASE + "/tramites/processo");
		httpPost.setHeader("Content-Type", "application/json");
		
		StringBuilder strRequest = new StringBuilder();
		strRequest.append("{");
		strRequest.append("    \"cabecalho\": {");
		strRequest.append("        \"remetente\": {");
		strRequest.append("            \"identificacaoDoRepositorioDeEstruturas\": \"1\",");
		strRequest.append("            \"numeroDeIdentificacaoDaEstrutura\": \"32288\"");//10399
		strRequest.append("        },");
		strRequest.append("        \"destinatario\": {");
		strRequest.append("            \"identificacaoDoRepositorioDeEstruturas\": \"1\",");
		strRequest.append("            \"numeroDeIdentificacaoDaEstrutura\": \"32291\"");//10400
		strRequest.append("        },");
		strRequest.append("        \"obrigarEnvioDeTodosOsComponentesDigitais\": true");
		strRequest.append("    },");
		strRequest.append("    \"processo\": {");
		strRequest.append("        \"protocolo\": \"123456\",");
		strRequest.append("        \"nivelDeSigilo\": 1,");
		strRequest.append("        \"produtor\": {");
		strRequest.append("            \"nome\": \"Ministério ABC\",");
		strRequest.append("            \"tipo\": \"orgaopublico\"");
		strRequest.append("        },");
		strRequest.append("        \"descricao\": \"Descrição ABC\",");
		strRequest.append("        \"dataHoraDeProducao\": \"2014-06-01T14:30:00-03:00\",");
		strRequest.append("        \"dataHoraDeRegistro\": \"2014-06-01T14:30:00-03:00\",");
		strRequest.append("        \"documentos\": [");
		strRequest.append("            {");
		strRequest.append("                \"ordem\": 1,");
		strRequest.append("                \"nivelDeSigilo\": 1,");
		strRequest.append("                \"produtor\": {");
		strRequest.append("                    \"nome\": \"Ministério ABC\",");
		strRequest.append("                    \"tipo\": \"orgaopublico\"");
		strRequest.append("                },");
		strRequest.append("                \"descricao\": \"Descrição ABC\",");
		strRequest.append("                \"dataHoraDeProducao\": \"2014-06-01T14:30:00-03:00\",");
		strRequest.append("                \"dataHoraDeRegistro\": \"2014-06-01T14:30:00-03:00\",");
		strRequest.append("                \"especie\": {");
		strRequest.append("                    \"codigo\": \"42\",");
		strRequest.append("                    \"nomeNoProdutor\": \"Despacho\"");
		strRequest.append("                },");
		strRequest.append("                \"componentesDigitais\": [");
		strRequest.append("                    {");
		strRequest.append("                        \"nome\": \"binario1.txt\",");
		strRequest.append("                        \"hash\": {");
		strRequest.append("                            \"algoritmo\": \"SHA256\",");
		strRequest.append("                            \"conteudo\": \"8/jWfz773OQKSMf6JW2yNwgHB7N4hNUskfEi4rjnXK8=\"");
		strRequest.append("                        },");
		strRequest.append("                        \"tipoDeConteudo\": \"txt\",");
		strRequest.append("                        \"mimeType\": \"text/plain\",");
		strRequest.append("                        \"tamanhoEmBytes\": 157,");
		strRequest.append("                        \"ordem\": 1,");
		strRequest.append("						   \"assinaturasDigitais\": [");
		strRequest.append("  							{");
		strRequest.append("                           		\"dataHora\": \"2014-06-01T14:30:00-03:00\",");
		strRequest.append("                          		\"cadeiaDoCertificado\": {");
		strRequest.append("          							\"formato\": \"PKCS7\",");
		strRequest.append("     							    \"conteudo\": \"vazio\"");
		strRequest.append("        							},");
		strRequest.append("    								\"hash\": {");
		strRequest.append("        								\"algoritmo\": \"SHA256withRSA\",");
		strRequest.append("       								\"conteudo\": \"vazio\"");
		strRequest.append("          						},");
		strRequest.append("         					\"razao\": \"Documento assinado...\",");
		strRequest.append("       						\"observacao\": \"A autenticidade...\"");
		strRequest.append("    						  }");
		strRequest.append("    					   ]");
		strRequest.append("                    }");
		strRequest.append("                ]");
		strRequest.append("            }");
		strRequest.append("        ],");
		strRequest.append("        \"interessados\": [");
		strRequest.append("            {");
		strRequest.append("                \"nome\": \"Pessoa ABC\"");
		strRequest.append("            }");
		strRequest.append("        ]");
		strRequest.append("    }");
		strRequest.append("}");
		httpPost.setEntity(new StringEntity(strRequest.toString()));
		
		CloseableHttpResponse response = httpclient.execute(httpPost);

		try {
		    System.out.println(response.getStatusLine());
		    
		    String strResponse = EntityUtils.toString(response.getEntity());
		    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
		    	System.out.println("Envio do processo: " + strResponse);
		    	JSONObject jsonObject = new JSONObject(strResponse);
		    	return jsonObject;
		    } else {
		    	throw new RuntimeException(strResponse);
		    }
		    
		} finally {
		    response.close();
		}
		
	}
	
}
